@@ -1,6 +1,6 @@
 use std::env::{join_paths, split_paths};
 use std::ops::Deref;
-use std::path::{Path, PathBuf};
+use std::path::PathBuf;
 
 use console::truncate_str;
 use eyre::Result;
@@ -10,9 +10,10 @@ use crate::config::{Config, Settings};
 use crate::direnv::DirenvDiff;
 use crate::env::{PATH_KEY, TERM_WIDTH, __MISE_DIFF};
 use crate::env_diff::{EnvDiff, EnvDiffOperation};
+use crate::hook_env::WatchFilePattern;
 use crate::shell::{get_shell, ShellType};
 use crate::toolset::{Toolset, ToolsetBuilder};
-use crate::{env, hook_env};
+use crate::{dirs, env, hook_env, hooks, watch_files};
 
 /// [internal] called by activate hook to update env vars directory change
 #[derive(Debug, clap::Args)]
@@ -36,7 +37,7 @@ impl HookEnv {
         let config = Config::try_get()?;
         let watch_files = config.watch_files()?;
         time!("hook-env");
-        if hook_env::should_exit_early(&watch_files) {
+        if hook_env::should_exit_early(watch_files.clone()) {
             return Ok(());
         }
         time!("should_exit_early");
@@ -58,11 +59,14 @@ impl HookEnv {
         let settings = Settings::try_get()?;
         patches.extend(self.build_path_operations(&settings, &paths, &__MISE_DIFF.path)?);
         patches.push(self.build_diff_operation(&diff)?);
-        patches.push(self.build_watch_operation(&watch_files)?);
+        patches.push(self.build_watch_operation(watch_files.clone())?);
+        patches.push(self.build_dir_operation()?);
 
         let output = hook_env::build_env_commands(&*shell, &patches);
         miseprint!("{output}")?;
         self.display_status(&config, &ts)?;
+        hooks::run_all_hooks(&ts);
+        watch_files::execute_runs(&ts);
 
         Ok(())
     }
@@ -74,7 +78,7 @@ impl HookEnv {
                 .list_current_installed_versions()
                 .into_iter()
                 .rev()
-                .map(|(_, v)| v.short().to_string())
+                .map(|(_, tv)| format!("{}@{}", tv.short(), tv.version))
                 .collect_vec();
             if !installed_versions.is_empty() {
                 let status = installed_versions.into_iter().rev().join(" ");
@@ -167,9 +171,19 @@ impl HookEnv {
         ))
     }
 
+    fn build_dir_operation(&self) -> Result<EnvDiffOperation> {
+        Ok(EnvDiffOperation::Add(
+            "__MISE_DIR".into(),
+            dirs::CWD
+                .as_ref()
+                .map(|p| p.to_string_lossy().to_string())
+                .unwrap_or_default(),
+        ))
+    }
+
     fn build_watch_operation(
         &self,
-        watch_files: impl IntoIterator<Item = impl AsRef<Path>>,
+        watch_files: impl IntoIterator<Item = WatchFilePattern>,
     ) -> Result<EnvDiffOperation> {
         let watches = hook_env::build_watches(watch_files)?;
         Ok(EnvDiffOperation::Add(
@@ -186,15 +200,3 @@ fn patch_to_status(patch: EnvDiffOperation) -> String {
         EnvDiffOperation::Remove(k) => format!("-{}", k),
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use crate::test::reset;
-    use test_log::test;
-
-    #[test]
-    fn test_hook_env() {
-        reset();
-        assert_cli!("hook-env", "--status", "-s", "fish");
-    }
-}
